// Copyright (C) 2009 Andrey Gruber (aka lamer)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "notes.h"
#include "login.h"
#include "encryption.h"

static BOOL CALLBACK Login_DlgProc (HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
static BOOL Login_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static void Login_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void Login_OnDestroy(HWND hwnd);
static BOOL IsPasswordValid(HWND hwnd);
static void EnableOKButton(HWND hwnd);

static int				m_DlgId;
static wchar_t			m_LangFile[MAX_PATH];
static wchar_t			m_InvalidPassword[256], m_InvalidOldPassword[256], m_BothPasswords[256];
static wchar_t			*lpNewPwrd;

int ShowLoginDialog(HINSTANCE hInstance, HWND hParent, int idDlg, wchar_t * pNewPwrd){
	m_DlgId = idDlg;
	wcscpy(m_LangFile, g_NotePaths.LangDir);
	wcscat(m_LangFile, g_NotePaths.LangFile);
	lpNewPwrd = pNewPwrd;
	if(lpNewPwrd)
		*lpNewPwrd = '\0';
	return DialogBoxParamW(hInstance, MAKEINTRESOURCEW(idDlg), hParent, Login_DlgProc, 0);
}

static BOOL CALLBACK Login_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_INITDIALOG, Login_OnInitDialog);
	HANDLE_MSG (hwnd, WM_COMMAND, Login_OnCommand);
	HANDLE_MSG (hwnd, WM_DESTROY, Login_OnDestroy);

	default: return FALSE;
	}
}

static void Login_OnDestroy(HWND hwnd)
{
	lpNewPwrd = NULL;
	g_hLoginDialog = NULL;
	if(m_DlgId == DLG_LOGIN_MAIN){
		// DISABLEINPYT		di;

		// g_StopInput = FALSE;
		// di.flag = TRUE;
		// di.hExclude = hwnd;
		// EnumThreadWindows(GetCurrentThreadId(), DisableEnableProc, (LPARAM)&di);
	}
}

static void Login_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch(id){
		case IDOK:
			if(IsPasswordValid(hwnd)){
				EndDialog(hwnd, IDOK);
			}
			break;
		case IDCANCEL:
			EndDialog(hwnd, IDCANCEL);
			break;
		case IDC_EDT_PWRD_MAIN:
		case IDC_EDT_PWRD_SECOND:
		case IDC_EDT_PWRD_NEW:
			if(codeNotify == EN_CHANGE)
				EnableOKButton(hwnd);
			break;

	}
}

static BOOL Login_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	wchar_t 		szBuffer[256];

	g_hLoginDialog = hwnd;
	GetPNotesMessage(g_NotePaths.LangDir, g_NotePaths.LangFile, L"invalid_pwrd", L"Incorrect password", m_InvalidPassword);
	GetPNotesMessage(g_NotePaths.LangDir, g_NotePaths.LangFile, L"invalid_old_pwrd", L"Incorrect old password", m_InvalidOldPassword);
	GetPNotesMessage(g_NotePaths.LangDir, g_NotePaths.LangFile, L"both_pwrd", L"Both password strings should be identical. Please, check the spelling.", m_BothPasswords);
	switch(m_DlgId){
		case DLG_LOGIN_MAIN:{
			// DISABLEINPYT		di;
			
			// di.flag = FALSE;
			// di.hExclude = hwnd;
			// EnumThreadWindows(GetCurrentThreadId(), DisableEnableProc, (LPARAM)&di);
			// g_StopInput = TRUE;

			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_ENTER_PWRD, L"Enter password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_MAIN, szBuffer);
			GetPrivateProfileStringW(L"captions", L"enter_pwrd", L"Enter Password", szBuffer, 256, m_LangFile);
			break;
		}
		case DLG_LOGIN_CREATE:
			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_ENTER_PWRD, L"Enter password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_MAIN, szBuffer);
			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_CONF_PWRD, L"Confirm password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_SECOND, szBuffer);
			GetPrivateProfileStringW(L"captions", L"create_pwrd", L"Password Setting", szBuffer, 256, m_LangFile);
			break;
		case DLG_LOGIN_CHANGE:
			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_OLD_PWRD, L"Old password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_MAIN, szBuffer);
			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_NEW_PWRD, L"New password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_NEW, szBuffer);
			GetPrivateProfileStringW(S_LOGIN_CAPTIONS, IK_CONF_PWRD, L"Confirm password", szBuffer, 256, m_LangFile);
			SetDlgItemTextW(hwnd, IDC_ST_PWRD_SECOND, szBuffer);
			GetPrivateProfileStringW(L"captions", L"change_pwrd", L"Password Changing", szBuffer, 256, m_LangFile);
			break;
		default:
			break;
	}
	SetWindowTextW(hwnd, szBuffer);
	// SetForegroundWindow(hwnd);
	SetFocus(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN));
	return FALSE;
}

static BOOL IsPasswordValid(HWND hwnd){
	wchar_t			szPwrdMain[256], szPwrdNew[256], szPwrdConfirm[256], szHash[256], szTemp[256];

	switch(m_DlgId){
		case DLG_LOGIN_MAIN:
			GetDlgItemTextW(hwnd, IDC_EDT_PWRD_MAIN, szPwrdMain, 256);
			CreateHashString(szHash, szPwrdMain);
			GetPrivateProfileStringW(S_NOTE, IK_PASSWORD, L"N/A", szTemp, 256, g_NotePaths.INIFile);
			if(wcscmp(szHash, szTemp) == 0)
				return TRUE;
			else{
				MessageBoxW(hwnd, m_InvalidPassword, 0, MB_OK | MB_ICONERROR);
				SendDlgItemMessageW(hwnd, IDC_EDT_PWRD_MAIN, EM_SETSEL, 0, -1);
			}
			break;
		case DLG_LOGIN_CREATE:
			GetDlgItemTextW(hwnd, IDC_EDT_PWRD_MAIN, szPwrdMain, 256);
			GetDlgItemTextW(hwnd, IDC_EDT_PWRD_SECOND, szPwrdConfirm, 256);
			if(wcscmp(szPwrdMain, szPwrdConfirm) == 0){
				CreateHashString(szHash, szPwrdMain);
				if(lpNewPwrd)
					wcscpy(lpNewPwrd, szHash);
				// WritePrivateProfileStringW(S_NOTE, IK_PASSWORD, szHash, g_NotePaths.INIFile);
				return TRUE;
			}
			else{
				MessageBoxW(hwnd, m_BothPasswords, 0, MB_OK | MB_ICONERROR);
				SetFocus(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN));
				SendDlgItemMessageW(hwnd, IDC_EDT_PWRD_MAIN, EM_SETSEL, 0, -1);
			}
			break;
		case DLG_LOGIN_CHANGE:
			GetDlgItemTextW(hwnd, IDC_EDT_PWRD_MAIN, szPwrdMain, 256);
			CreateHashString(szHash, szPwrdMain);
			GetPrivateProfileStringW(S_NOTE, IK_PASSWORD, L"N/A", szTemp, 256, g_NotePaths.INIFile);
			if(wcscmp(szHash, szTemp) == 0){
				GetDlgItemTextW(hwnd, IDC_EDT_PWRD_NEW, szPwrdNew, 256);
				GetDlgItemTextW(hwnd, IDC_EDT_PWRD_SECOND, szPwrdConfirm, 256);
				if(wcscmp(szPwrdNew, szPwrdConfirm) == 0){
					CreateHashString(szHash, szPwrdNew);
					//store hash of new password
					if(lpNewPwrd)
						wcscpy(lpNewPwrd, szHash);
					// WritePrivateProfileStringW(S_NOTE, IK_PASSWORD, szHash, g_NotePaths.INIFile);
					return TRUE;
				}
				else{
					MessageBoxW(hwnd, m_BothPasswords, 0, MB_OK | MB_ICONERROR);
					SetFocus(GetDlgItem(hwnd, IDC_EDT_PWRD_NEW));
					SendDlgItemMessageW(hwnd, IDC_EDT_PWRD_NEW, EM_SETSEL, 0, -1);
				}
			}
			else{
				MessageBoxW(hwnd, m_InvalidOldPassword, 0, MB_OK | MB_ICONERROR);
				SetFocus(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN));
				SendDlgItemMessageW(hwnd, IDC_EDT_PWRD_MAIN, EM_SETSEL, 0, -1);
			}
			break;
	}
	return FALSE;
}

static void EnableOKButton(HWND hwnd){
	switch(m_DlgId){
		case DLG_LOGIN_MAIN:
			if(GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN)) > 0)
				EnableWindow(GetDlgItem(hwnd, IDOK), TRUE);
			else
				EnableWindow(GetDlgItem(hwnd, IDOK), FALSE);
			break;
		case DLG_LOGIN_CREATE:
			if(GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN)) > 0 && GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_SECOND)) > 0)
				EnableWindow(GetDlgItem(hwnd, IDOK), TRUE);
			else
				EnableWindow(GetDlgItem(hwnd, IDOK), FALSE);
			break;
		case DLG_LOGIN_CHANGE:
			if(GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_MAIN)) > 0 && GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_SECOND)) > 0 && GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_PWRD_NEW)) > 0)
				EnableWindow(GetDlgItem(hwnd, IDOK), TRUE);
			else
				EnableWindow(GetDlgItem(hwnd, IDOK), FALSE);
			break;
	}
}
